package me.sergiotarxz.bedrockstation;

import android.os.Bundle;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import android.widget.LinearLayout;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.view.View;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Gravity;
import android.database.sqlite.SQLiteDatabase;
import android.view.ViewGroup.LayoutParams;
import android.content.Intent;
import me.sergiotarxz.bedrockstation.ProxyService;
import me.sergiotarxz.bedrockstation.Options;
import me.sergiotarxz.bedrockstation.DB;
import android.widget.Toast;
import androidx.activity.result.contract.ActivityResultContracts.RequestPermission;
import androidx.activity.result.ActivityResultLauncher;
import android.app.NotificationManager;
import android.content.ServiceConnection;
import android.content.Context;
import android.content.ComponentName;
import android.os.IBinder;
import android.util.TypedValue;
import java.util.List;
import me.sergiotarxz.bedrockstation.Servers;
import me.sergiotarxz.bedrockstation.Server;
import android.app.AlertDialog;
import android.content.DialogInterface;
import me.sergiotarxz.bedrockstation.Premium;

public class MainActivity extends AppCompatActivity {
    interface Lambda {
	void l();
    };

    private Options options;

    private LinearLayout serverListInnerLayout = null;
    ProxyService proxyService = null;
    boolean mBound = false;
    private Button directConnectionButton;
    boolean serverStarted = false;
    private EditText hostEditText = null;
    private EditText portEditText = null;

    private ServiceConnection connection = new ServiceConnection() {
	@Override
	public void onServiceConnected(ComponentName className,
		IBinder service) {
	    ProxyService.LocalBinder binder = (ProxyService.LocalBinder) service;
	    proxyService = binder.getService();
            if (proxyService.isServerStarted()) {
                onStartProxyService();
            } else {
                onFinishProxyService();
            }
            proxyService.setActivity(MainActivity.this);
	    mBound = true;
	}

	@Override
	public void onServiceDisconnected(ComponentName arg0) {
	    mBound = false;
            onFinishProxyService();
	}
    };

    private void onChangeEditText(EditText edit, Lambda l) {
	edit.addTextChangedListener(new TextWatcher() {
		
            public void afterTextChanged(Editable s) {
		l.l();
            }

            public void beforeTextChanged(CharSequence s, int start, int count, int after) {}

            public void onTextChanged(CharSequence s, int start, int before, int count) {}
	});
    }

    
    private void finishServiceProxy() {
        Intent intent = new Intent(this, ProxyService.class);
        intent.setAction(ProxyService.Action.END);
        this.startForegroundService(intent);
        try {
            Thread.sleep(250);
        } catch (Exception e) {
        }
    }

    private void startServiceProxy() {
        Intent intent = new Intent(this, ProxyService.class);
        this.startForegroundService(intent);
    }

    @Override
    protected void onStart() {
        super.onStart();
        Intent intent = new Intent(this, ProxyService.class);
        bindService(intent, connection, Context.BIND_AUTO_CREATE);
    }

    @Override
    protected void onStop() {
        super.onStop();
        unbindService(connection);

    }

    public void onStartProxyService() {
        directConnectionButton.setText("Finish Proxy");
        serverStarted = true;
    }

    public void onFinishProxyService() {
        directConnectionButton.setText("Direct connection");
        serverStarted = false;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        LinearLayout layout = new LinearLayout(this);
	layout.setOrientation(LinearLayout.VERTICAL);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(
            LayoutParams.MATCH_PARENT,
            LayoutParams.MATCH_PARENT);
        layoutParams.setMargins(100, 100, 100, 100);
        layout.setLayoutParams(layoutParams);
        layout.setId(1);
        SQLiteDatabase db = new DB(this).getInstance();
        options = new Options(db);
        Servers servers = Servers.instance(db);
        addDirectConnection(layout);
        addServerList(layout, servers);
        setContentView(layout);
    }

    private void addDirectConnection(LinearLayout layout) {
        TextView header = new TextView(this);
        header.setTextSize(TypedValue.COMPLEX_UNIT_PX, 70);
        header.setText("Connect without saving");

        directConnectionButton = new Button(this);
        directConnectionButton.setOnClickListener( (View view) -> {
            options.set(Options.HOST_SELECTED, options.get(Options.HOST_CACHE));
            options.set(Options.PORT_SELECTED, options.get(Options.PORT_CACHE));
            if(!proxyService.isServerStarted()) {
                if (!getSystemService(NotificationManager.class)
                    .areNotificationsEnabled()) {
                    requestPermissionLauncher.launch(android.Manifest.permission.POST_NOTIFICATIONS);
                    return;
                }
                if (Premium.ENABLED) {
                    startServiceProxy();
                    return;
                }
                Intent intent = new Intent(this, CountDownStartProxy.class);
                startActivity(intent);
                return;
            }
            finishServiceProxy();
        });
        if (serverStarted) {
            directConnectionButton.setText("Finish Proxy");
        } else {
            directConnectionButton.setText("Start Proxy");
        }
        LayoutParams buttonLayoutParams = new LinearLayout.LayoutParams(
            LayoutParams.WRAP_CONTENT,
            LayoutParams.WRAP_CONTENT);
        LayoutParams editTextLayoutParams = new LinearLayout.LayoutParams(
            LayoutParams.WRAP_CONTENT,
            LayoutParams.WRAP_CONTENT);
	editTextLayoutParams.width = 500;
        directConnectionButton.setLayoutParams(buttonLayoutParams);
	LinearLayout hostLayout = new LinearLayout(this);
	TextView hostIndicator = new TextView(this);
	hostIndicator.setText("IP: ");
        hostEditText = new EditText(this);
        hostEditText.setText(options.get(Options.HOST_CACHE));
	onChangeEditText(hostEditText, () -> {
	    String text = hostEditText.getText().toString();
            options.set(Options.HOST_CACHE, text);
	});
        hostEditText.setLayoutParams(editTextLayoutParams);
	hostLayout.addView(hostIndicator);
	hostLayout.addView(hostEditText);
	LinearLayout portLayout = new LinearLayout(this);
	TextView portIndicator = new TextView(this);
        portEditText = new EditText(this);
       	onChangeEditText(portEditText, () -> {
	    String text = portEditText.getText().toString();
            options.set(Options.PORT_CACHE, text);
	});
        portEditText.setLayoutParams(editTextLayoutParams);
	portIndicator.setText("Port: ");
        portEditText.setText(options.get(Options.PORT_CACHE));
	portLayout.addView(portIndicator);
	portLayout.addView(portEditText);
        center(header);
        layout.addView(header);
        layout.addView(hostLayout);
        layout.addView(portLayout);
        layout.addView(directConnectionButton);
    }

    private void addServerList(LinearLayout layout, Servers servers) {
        LinearLayout serverListLayout = new LinearLayout(this);
        serverListLayout.setOrientation(LinearLayout.VERTICAL);
        center(serverListLayout);
        TextView labelServerList = new TextView(this);
        center(labelServerList);
        labelServerList.setText("List of added servers");
        labelServerList.setTextSize(TypedValue.COMPLEX_UNIT_PX, 70);
        serverListLayout.addView(labelServerList);
        fillServerList(serverListLayout, servers);
        layout.addView(serverListLayout);
        addCreateNewServerForm(layout, servers);
    }

    private void addCreateNewServerForm(LinearLayout layout, Servers servers) {
        LinearLayout form = new LinearLayout(this);
        center(form);
        form.setOrientation(LinearLayout.VERTICAL);
        TextView header = new TextView(this);
        header.setText("Create a new server");
        header.setTextSize(TypedValue.COMPLEX_UNIT_PX, 70);
        center(header);
        LayoutParams editTextLayoutParams = new LinearLayout.LayoutParams(
            LayoutParams.WRAP_CONTENT,
            LayoutParams.WRAP_CONTENT);
	editTextLayoutParams.width = 500;

        LinearLayout titleLayout = new LinearLayout(this);
        TextView titleLabel = new TextView(this);
        EditText titleEdit = new EditText(this);
        titleEdit.setLayoutParams(editTextLayoutParams);
        titleLabel.setText("Server name: ");
        titleLayout.addView(titleLabel);
        titleLayout.addView(titleEdit);
        center(titleLayout);

        LinearLayout hostLayout = new LinearLayout(this);
        TextView hostLabel = new TextView(this);
        EditText hostEdit = new EditText(this);
        hostEdit.setLayoutParams(editTextLayoutParams);
        hostLabel.setText("Host: ");
        hostLayout.addView(hostLabel);
        hostLayout.addView(hostEdit);
        center(hostLayout);

        LinearLayout portLayout = new LinearLayout(this);
        TextView portLabel = new TextView(this);
        EditText portEdit = new EditText(this);
        portEdit.setLayoutParams(editTextLayoutParams);
        portLabel.setText("Port: ");
        portLayout.addView(portLabel);
        portLayout.addView(portEdit);
        center(portLayout);
        Button create = new Button(this);
        create.setOnClickListener( (View view) -> {
            if (!Premium.ENABLED) {
                alertSomethingWrong("Payment required", "You need premium to add new servers");
                return;
            }
            int port; 
            String title;
            String host;
            try {
                port = Integer.parseInt(portEdit.getText().toString());
            } catch (Exception e) {
                alertSomethingWrong("Invalid data", "Port must be a number");
                return;
            }
            title = titleEdit.getText().toString();
            host = hostEdit.getText().toString();
            if (host.indexOf("\n") != -1 || host.equals("") || host.indexOf(" ") != -1) {
                alertSomethingWrong("Invalid data", "Not a valid host");
                return;
            }
            if (title.indexOf("\n") != -1) {
                alertSomethingWrong("Invalid data", "Not a valid title");
                return;
            }
            servers.add(new Server(title, host, port));
            titleEdit.setText("");
            hostEdit.setText("");
            portEdit.setText("");
            fillServerList(null, servers);
        });
        create.setText("Add this server");

        form.addView(header);
        form.addView(titleLayout);
        form.addView(hostLayout);
        form.addView(portLayout);
        form.addView(create);
        layout.addView(form); 
    }

    public void fillServerList(LinearLayout serverListLayout, Servers servers) {
        boolean firstSetup = false;
        if (serverListInnerLayout == null) {
            firstSetup = true;
            serverListInnerLayout = new LinearLayout(this);
            serverListInnerLayout.setOrientation(LinearLayout.VERTICAL);
        }
        do {
            List<Server> serverList = servers.all();
            serverListInnerLayout.removeViews(0, serverListInnerLayout.getChildCount());
            if (serverList.size() < 1) {
                TextView nothingHereYet = new TextView(this);
                nothingHereYet.setText("You did not add any servers yet.");
                center(nothingHereYet);
                serverListInnerLayout.addView(nothingHereYet);
                break;
            }
            for (Server server : serverList) {
                TextView title = new TextView(this);
                title.setTextSize(TypedValue.COMPLEX_UNIT_PX, 60);
                title.setText(server.getTitle());
                TextView host = new TextView(this);
                host.setText(server.getHost() + ":" + server.getPort());
                serverListInnerLayout.addView(title);
                serverListInnerLayout.addView(host);
                LinearLayout buttons = new LinearLayout(this);
                Button connect = new Button(this);
                connect.setText("Connect");
                Button delete = new Button(this);
                delete.setText("Delete");
                buttons.addView(connect);
                connect.setOnClickListener( (View view) -> {
                    if (serverStarted) {
                        finishServiceProxy();
                    }
                    options.set(Options.HOST_SELECTED, server.getHost());
                    options.set(Options.PORT_SELECTED, ""+server.getPort());
                    startServiceProxy();
                });
                delete.setOnClickListener( (View view) -> {
                new AlertDialog.Builder(this)
                    .setTitle("Delete server")
                    .setMessage("Are you sure you want to delete this server?")
                    .setPositiveButton("Delete server", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) { 
                            servers.delete(server);
                            fillServerList(null, servers);
                        }
                     })
                    .setNegativeButton("Keep server", null)
                    .show();
                });
                buttons.addView(delete);
                serverListInnerLayout.addView(buttons);
            }
        } while (false);
        if (firstSetup) {
            serverListLayout.addView(serverListInnerLayout);
        }
    }

    private void center(TextView view) {
    }

    private void center(LinearLayout view) {
    }

    private void alertSomethingWrong(String title, String message) {
        new AlertDialog.Builder(this)
        .setTitle(title)
        .setMessage(message)

        .setPositiveButton("Accept", null)
        .show();
    }

    private ActivityResultLauncher<String> requestPermissionLauncher =
        registerForActivityResult(new RequestPermission(), isGranted -> {
            if (isGranted) {
                if (Premium.ENABLED) {
                    startServiceProxy();
                    return;
                }
                Intent intent = new Intent(this, CountDownStartProxy.class);
                startActivity(intent);
            } else {
                alertSomethingWrong("Notifications not enabled", "You need notifications to run the proxy");
            }
    });

}
