package me.sergiotarxz.bedrockstation;

import android.util.Log;
import androidx.core.app.ServiceCompat;
import androidx.core.app.NotificationCompat;
import android.content.pm.ServiceInfo;
import android.os.Build;
import android.os.Binder;
import android.os.IBinder;
import android.app.Service;
import android.app.Notification;
import android.content.Intent;
import android.app.NotificationManager;
import android.app.NotificationChannel;
import java.net.InetSocketAddress;
import android.app.PendingIntent;
import android.database.sqlite.SQLiteDatabase;
import me.sergiotarxz.bedrockstation.DB;
import android.widget.Toast;
import android.os.StrictMode;
import android.app.AlertDialog;

import me.sergiotarxz.bedrockstation.ProxyThread;

public class ProxyService extends Service {

    static String CHANNEL_ID = "sthaoes";
    Thread proxyThread = null;

    public class LocalBinder extends Binder {
        public ProxyService getService() {
            return ProxyService.this;
        }
    }

    private final IBinder binder = new LocalBinder();

    private MainActivity activity = null;

    public void setActivity(MainActivity activity) {
        this.activity = activity;
    }

    @Override
    public IBinder onBind(Intent intent) {
        return binder;
    }

    static ProxyService instance = null;
    static public ProxyService getInstance() {
        return instance;
    }

    public class Action {
        public static final String START = "start";
        public static final String END = "end";
    }

    @Override
    public void onCreate() {
        NotificationManager notificationManager = getSystemService(NotificationManager.class);
        NotificationChannel channel = new NotificationChannel(CHANNEL_ID, "BedrockProxy", NotificationManager.IMPORTANCE_DEFAULT);
        notificationManager.createNotificationChannel(channel);
        instance = this;
    }

    private void startForeground() {
        try {
            Intent intent = new Intent(this, ProxyService.class);
            intent.setAction(Action.END);
            PendingIntent pendingIntent = PendingIntent.getForegroundService(this, 100, intent, PendingIntent.FLAG_IMMUTABLE);
            Notification notification = new NotificationCompat.Builder(this, CHANNEL_ID)
                .setContentTitle("Bedrock Proxy is running")
                .setContentText("Press this notification to kill")
                .setSmallIcon(R.drawable.ic_launcher_foreground)
                .setContentIntent(pendingIntent)
                .setOngoing(true)
                .build();
            int type = 0;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                type = ServiceInfo.FOREGROUND_SERVICE_TYPE_DATA_SYNC;
            }
            this.startForeground(
                1,
                notification,
                type
            );
        } catch (Exception e) {
            throw new RuntimeException(e.toString());
        }
    }

    @Override
    public int onStartCommand(Intent intent,
        int flags,
        int startId) {
        if (intent.getAction() == null) {
            intent.setAction(Action.START);
        }
        if (intent.getAction() == Action.START) {
            startForeground();
            startServer();
        }
        if (intent.getAction() == Action.END) {
            finishServer();
            stopForeground(true);
        }
        return super.onStartCommand(intent, flags, startId);
    }

    public boolean isServerStarted() {
        return proxyThread != null && proxyThread.isAlive();
    }

    public void finishServer() {
        if (isServerStarted()) {
            Log.e("bedrockstation", "FINISHING SERVER");
            ((ProxyThread) proxyThread).terminate();
            try {
                proxyThread.join();
                activity.onFinishProxyService();
            } catch (Exception e) {
                throw new RuntimeException(e.toString());
            }
        }
    }

    private void createServer(InetSocketAddress address) {
        try {
            proxyThread = new ProxyThread(address);
        } catch (Exception e) {
            throw new RuntimeException(e.toString());
        }
    }
    private void alertSomethingWrong(String title, String message) {
//            if (!isFinishing()) {
//                new AlertDialog.Builder(this)
//                .setTitle(title)
//                .setMessage(message)
//
//                // Specifying a listener allows you to take an action before dismissing the dialog.
//                // The dialog is automatically dismissed when a dialog button is clicked.
//                .setPositiveButton("accept", null)
//                .show();
//            }
    }

    public void startServer() {
        if (!isServerStarted()) {
            Log.e("bedrockstation", "STARTING SERVER");
            Options options = new Options(new DB(this).getInstance());
            String host = options.get(Options.HOST_SELECTED);
            
            int port = 0;
            try {
                port = Integer.parseInt(options.get(Options.PORT_SELECTED));
            } catch (Exception e) {
                Log.e("bedrockstation", Log.getStackTraceString(e));
                Toast.makeText(this, "Port is not a number", Toast.LENGTH_LONG).show();
                return;
            }
            StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
            StrictMode.setThreadPolicy(policy);
            try {
                createServer(new InetSocketAddress(host, port));
                proxyThread.start();
                activity.onStartProxyService();
            } catch (Exception e) {
                alertSomethingWrong("Invalid server address", "The server address is malformed");
            }
        }
    }
}
