package me.sergiotarxz.bedrockstation;

import java.nio.channels.DatagramChannel;
import java.nio.channels.Selector;
import java.nio.channels.SelectableChannel;
import java.util.IdentityHashMap;
import java.util.HashMap;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.ByteBuffer;
import java.util.Set;
import java.net.StandardSocketOptions;
import java.util.Iterator;
import java.lang.Thread;
import android.util.Log;

interface Lambda {
    void l() throws Exception;
};

public class ProxyThread extends Thread
{
    IdentityHashMap<Object, Lambda> objectToFunc;
    HashMap<String, DatagramChannel> addressToServer;
    Selector selector;
    DatagramChannel server;
    InetSocketAddress remoteServerAddress;


    private volatile boolean running = true;

    public void terminate() {
        running = false;
    }

    public ProxyThread(InetSocketAddress remoteServerAddress) throws Exception {
        this.remoteServerAddress = remoteServerAddress;
        objectToFunc = new IdentityHashMap<Object, Lambda>();
        addressToServer = new HashMap<String, DatagramChannel>();
        selector = Selector.open();
    }

    @Override
    public void run() {
        try {
            server = createServer();
            while (running) {
                selector.select(1000);
                Set<SelectionKey> selectedKeys = selector.selectedKeys();
                Iterator<SelectionKey> iter = selectedKeys.iterator();
                while (iter.hasNext()) {
                    SelectionKey key = iter.next();
                    SelectableChannel channel = key.channel();
                    if (!(channel instanceof DatagramChannel)) {
                        iter.remove();
                        continue;
                    }
                    DatagramChannel dchannel = (DatagramChannel) channel;
                    if (key.isReadable()) {
                        objectToFunc.get(dchannel).l();
                    }
                    iter.remove();
                }
            }
            server.close();
        } catch (Exception e) {
            Log.w("me.sergiotarxz.bedrockstation", Log.getStackTraceString(e));
        }
    }

    public DatagramChannel createServer() throws Exception {
            DatagramChannel datagramChannel = DatagramChannel.open();
            InetSocketAddress serverAddress = new InetSocketAddress("0.0.0.0", remoteServerAddress.getPort());
            datagramChannel.bind(serverAddress);
            datagramChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
            datagramChannel.configureBlocking(false);
            objectToFunc.put(datagramChannel, () -> {
                onServerCanRead(datagramChannel);
            });
            datagramChannel.register(selector, SelectionKey.OP_READ);
            System.out.println("Redirecting UDP connections from (" + "0.0.0.0" + ":" + remoteServerAddress.getPort() + ") to (" + remoteServerAddress.getHostString() + ":" + remoteServerAddress.getPort() + ")");
            return datagramChannel;
    }

    public DatagramChannel createClientFinalServer(SocketAddress finalClientAddress) throws Exception {
            DatagramChannel datagramChannel = DatagramChannel.open();
            InetSocketAddress serverAddress = remoteServerAddress;
            datagramChannel.bind(null);
            datagramChannel.configureBlocking(false);
            objectToFunc.put(datagramChannel, () -> {
                onClientFinalServerCanRead(datagramChannel, finalClientAddress);
            });
            datagramChannel.register(selector, SelectionKey.OP_READ);
            return datagramChannel;
    }

    public void onClientFinalServerCanRead(
                DatagramChannel finalServerConnection,
                SocketAddress finalClientAddress
    ) throws Exception {
        ByteBuffer buffer = ByteBuffer.allocate(1024);
        finalServerConnection.receive(buffer);
        byte[] array = buffer.array();
        int size = buffer.capacity() - (buffer.capacity() - buffer.position());
        server.send(ByteBuffer.wrap(array, 0, size), finalClientAddress);
    }

    public void onServerCanRead(
                DatagramChannel server
    ) throws Exception {
        ByteBuffer buffer = ByteBuffer.allocate(1024);
        SocketAddress clientAddress = server.receive(buffer);
        int size = buffer.capacity() - (buffer.capacity() - buffer.position());
        byte[] array = buffer.array();
        getClientFinalConnection(clientAddress)
            .send(ByteBuffer.wrap(array, 0, size), new InetSocketAddress("192.168.2.1", 19132));
    }

    public DatagramChannel getClientFinalConnection( SocketAddress clientAddress ) throws Exception {
        DatagramChannel connection = addressToServer.get(clientAddress.toString());
        if (connection != null) {
            return connection;
        }
        connection = createClientFinalServer(clientAddress);
        if (clientAddress instanceof InetSocketAddress) {
            InetSocketAddress clientAddressInet = (InetSocketAddress) clientAddress;
            System.out.println("New UDP Client at (" + clientAddressInet.getHostString() + ":" + clientAddressInet.getPort() + ")");
        }
        addressToServer.put(clientAddress.toString(), connection);
        return connection;
    }
}
