package me.sergiotarxz.bedrockstation;

import android.database.sqlite.SQLiteDatabase;
import me.sergiotarxz.bedrockstation.DBContract;
import me.sergiotarxz.bedrockstation.Server;
import android.database.Cursor;
import android.content.ContentValues;
import java.util.List;
import java.util.ArrayList;

public class Servers {
    private SQLiteDatabase db = null;

    private static Servers servers = null;

    public static Servers instance(SQLiteDatabase db) {
        if (servers != null) {
            return servers;
        }
        servers = new Servers(db);
        return servers;
    }

    public Servers(SQLiteDatabase db) {
        this.db = db;
    }

    public void delete(Server server) {
        String selection = DBContract.Servers.COLUMN_NAME_ID + " = ?";
        String[] selectionArgs = { server.getId()+"" };
        db.delete(
            DBContract.Servers.TABLE_NAME,
            selection,
            selectionArgs
        );
    }

    public void update_or_add(Server server) {
        if (server.getId() == null) {
            add(server);
            return;
        }
        update(server);
        return;
    }

    public void add(Server server) {
        ContentValues values = new ContentValues();
        values.put(DBContract.Servers.COLUMN_NAME_TITLE, server.getTitle());
        values.put(DBContract.Servers.COLUMN_NAME_HOST, server.getHost());
        values.put(DBContract.Servers.COLUMN_NAME_PORT, server.getPort());
        server.setId((int) db.insert(DBContract.Servers.TABLE_NAME, null, values));
    }

    public void update(Server server) {
        ContentValues values = new ContentValues();
        values.put(DBContract.Servers.COLUMN_NAME_TITLE, server.getTitle());
        values.put(DBContract.Servers.COLUMN_NAME_HOST, server.getHost());
        values.put(DBContract.Servers.COLUMN_NAME_PORT, server.getPort());

        String selection = DBContract.Servers.COLUMN_NAME_ID + " = ?";
        String[] selectionArgs = { server.getId()+"" };
        db.update(
            DBContract.Servers.TABLE_NAME,
            values,
            selection,
            selectionArgs
        );
    }

    public List<Server> all() {
        String[] projection = {
            DBContract.Servers.COLUMN_NAME_ID,
            DBContract.Servers.COLUMN_NAME_TITLE,
            DBContract.Servers.COLUMN_NAME_HOST,
            DBContract.Servers.COLUMN_NAME_PORT
        };
        String selection = "";
        String[] selectionArgs = {};
        Cursor cursor = db.query(
            DBContract.Servers.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            ""
        );
        ArrayList<Server> array = new ArrayList<Server>();
        while (cursor.moveToNext()) {
            int id = cursor.getInt(
                columnName(cursor, DBContract.Servers.COLUMN_NAME_ID)
            );
            String title = cursor.getString(
                columnName(cursor, DBContract.Servers.COLUMN_NAME_TITLE)
            );
            String host = cursor.getString(
                columnName(cursor, DBContract.Servers.COLUMN_NAME_HOST)
            );
            int port = cursor.getInt(
                columnName(cursor, DBContract.Servers.COLUMN_NAME_PORT)
            );
            array.add(new Server(id, title, host, port));
        }
        cursor.close();
        return array;
    }

    public Server get(int id) {
        String[] projection = {
            DBContract.Servers.COLUMN_NAME_ID,
            DBContract.Servers.COLUMN_NAME_TITLE,
            DBContract.Servers.COLUMN_NAME_HOST,
            DBContract.Servers.COLUMN_NAME_PORT
        };
        String selection = DBContract.Servers.COLUMN_NAME_ID + " = ?";
        String[] selectionArgs = { id+"" };
        Cursor cursor = db.query(
            DBContract.Servers.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            ""
        );
        if (!cursor.moveToNext()) {
            return null;
        }
        id = cursor.getInt(
            columnName(cursor, DBContract.Servers.COLUMN_NAME_ID)
        );
        String title = cursor.getString(
            columnName(cursor, DBContract.Servers.COLUMN_NAME_TITLE)
        );
        String host = cursor.getString(
            columnName(cursor, DBContract.Servers.COLUMN_NAME_HOST)
        );
        int port = cursor.getInt(
            columnName(cursor, DBContract.Servers.COLUMN_NAME_PORT)
        );
        cursor.close();
        return new Server(id, title, host, port);
    }

    public int columnName(Cursor cursor, String columnName) {
        return cursor.getColumnIndexOrThrow(columnName);
    }
}
